@@ -23,6 +23,7 @@
 #include <libgnome/gnome-i18n.h>
 #include <libedataserver/e-dbhash.h>
 #include <libedataserver/e-db3-utils.h>
+#include <libedataserver/e-util.h>
 #include <libebook/e-contact.h>
 
 #include <libedata-book/e-book-backend-sexp.h>
@@ -31,6 +32,8 @@
 #include <libedata-book/e-data-book-view.h>
 #include "e-book-backend-file.h"
 
+#define d(x)
+
 #define CHANGES_DB_SUFFIX ".changes.db"
 
 #define E_BOOK_BACKEND_FILE_VERSION_NAME "PAS-DB-VERSION"
@@ -129,9 +132,10 @@ do_create(EBookBackendFile  *bf,
 	if (0 == db_error) {
 		db_error = db->sync (db, 0);
 		if (db_error != 0)
-			g_warning ("db->sync failed.\n");
+			g_warning ("db->sync failed with %d", db_error);
 	}
 	else {
+		g_warning ("db->put failed with %d", db_error);
 		g_object_unref (contact);
 		contact = NULL;
 	}
@@ -168,7 +172,7 @@ e_book_backend_file_remove_contacts (EBookBackendSync *backend,
 {
 	EBookBackendFile *bf = E_BOOK_BACKEND_FILE (backend);
 	DB             *db = bf->priv->file_db;
-	DBT            id_dbt, vcard_dbt;
+	DBT            id_dbt;
 	int            db_error;
 	char          *id;
 	GList         *l;
@@ -179,14 +183,7 @@ e_book_backend_file_remove_contacts (EBookBackendSync *backend,
 		id = l->data;
 
 		string_to_dbt (id, &id_dbt);
-		memset (&vcard_dbt, 0, sizeof (vcard_dbt));
 
-		db_error = db->get (db, NULL, &id_dbt, &vcard_dbt, 0);
-		if (0 != db_error) {
-			rv = GNOME_Evolution_Addressbook_ContactNotFound;
-			continue;
-		}
-	
 		db_error = db->del (db, NULL, &id_dbt, 0);
 		if (0 != db_error) {
 			rv = GNOME_Evolution_Addressbook_ContactNotFound;
@@ -311,7 +308,7 @@ e_book_backend_file_get_contact_list (EBookBackendSync *backend,
 	const char *search = query;
 	GList *contact_list = NULL;
 
-	printf ("e_book_backend_file_get_contact_list (%s)\n", search);
+	d(printf ("e_book_backend_file_get_contact_list (%s)\n", search));
 
 	search_needed = TRUE;
 
@@ -361,11 +358,39 @@ typedef struct {
 	gboolean stopped;
 } FileBackendSearchClosure;
 
+static void
+closure_destroy (FileBackendSearchClosure *closure)
+{
+	d(printf ("destroying search closure\n"));
+	g_mutex_free (closure->mutex);
+	g_free (closure);
+}
+
+static FileBackendSearchClosure*
+init_closure (EDataBookView *book_view)
+{
+	FileBackendSearchClosure *closure = g_new (FileBackendSearchClosure, 1);
+
+	closure->mutex = g_mutex_new();
+	closure->stopped = FALSE;
+
+	g_object_set_data_full (G_OBJECT (book_view), "EBookBackendFile.BookView::closure",
+				closure, (GDestroyNotify)closure_destroy);
+
+	return closure;
+}
+
+static FileBackendSearchClosure*
+get_closure (EDataBookView *book_view)
+{
+	return g_object_get_data (G_OBJECT (book_view), "EBookBackendFile.BookView::closure");
+}
+
 static void
 e_book_backend_file_start_book_view (EBookBackend  *backend,
 				     EDataBookView *book_view)
 {
-	FileBackendSearchClosure *closure = g_new0 (FileBackendSearchClosure, 1);
+	FileBackendSearchClosure *closure;
 	EBookBackendFile *bf = E_BOOK_BACKEND_FILE (backend);
 	const char *query;
 	DB  *db;
@@ -373,19 +398,36 @@ e_book_backend_file_start_book_view (EBookBackend  *backend,
 	int db_error;
 	gboolean stopped = FALSE;
 
-	printf ("starting initial population of book view\n");
+	d(printf ("starting initial population of book view\n"));
+
+	g_mutex_lock (e_data_book_view_get_mutex (book_view));
+
+	closure = get_closure (book_view);
+	if (closure) {
+		/* the only way op can be set here is if we raced with
+		   stop_book_view and lost.  make sure by checking
+		   op->stopped. */
+		if (!closure->stopped) {
+			g_warning ("lost race with stop_book_view, but op->stopped != TRUE");
+		}
+		g_object_set_data (G_OBJECT (book_view), "EBookBackendFile.BookView::closure", NULL);
+		g_mutex_unlock (e_data_book_view_get_mutex (book_view));
+		return;
+	}
+	else {
+		closure = init_closure (book_view);
+		closure->stopped = FALSE;
+	}
 
 	/* ref the book view because it'll be removed and unrefed
 	   when/if it's stopped */
-	g_object_ref (book_view);
+	bonobo_object_ref (book_view);
 
+	g_mutex_unlock (e_data_book_view_get_mutex (book_view));
+	
 	db = bf->priv->file_db;
 	query = e_data_book_view_get_card_query (book_view);
 
-	closure->mutex = g_mutex_new();
-	closure->stopped = FALSE;
-	g_object_set_data (G_OBJECT (book_view), "EBookBackendFile.BookView::closure", closure);
-
 	if ( ! strcmp (query, "(contains \"x-evolution-any-field\" \"\")"))
 		e_data_book_view_notify_status_message (book_view, _("Loading..."));
 	else
@@ -403,11 +445,8 @@ e_book_backend_file_start_book_view (EBookBackend  *backend,
 			stopped = closure->stopped;
 			g_mutex_unlock (closure->mutex);
 
-			if (stopped) {
-				g_mutex_free (closure->mutex);
-				g_free (closure);
+			if (stopped)
 				break;
-			}
 
 			string_to_dbt (id, &id_dbt);
 			memset (&vcard_dbt, 0, sizeof (vcard_dbt));
@@ -439,11 +478,8 @@ e_book_backend_file_start_book_view (EBookBackend  *backend,
 			stopped = closure->stopped;
 			g_mutex_unlock (closure->mutex);
 
-			if (stopped) {
-				g_mutex_free (closure->mutex);
-				g_free (closure);
+			if (stopped)
 				break;
-			}
 
 			/* don't include the version in the list of cards */
 			if (strcmp (id_dbt.data, E_BOOK_BACKEND_FILE_VERSION_NAME)) {
@@ -465,37 +501,38 @@ e_book_backend_file_start_book_view (EBookBackend  *backend,
 
 	}
 
-	g_mutex_lock (closure->mutex);
-	stopped = closure->stopped;
-	g_mutex_unlock (closure->mutex);
 	if (!stopped)
 		e_data_book_view_notify_complete (book_view, GNOME_Evolution_Addressbook_Success);
 
-	g_mutex_free (closure->mutex);
-	g_free (closure);
-
-	g_object_set_data (G_OBJECT (book_view), "EBookBackendFile.BookView::closure", NULL);
-
 	/* unref the */
-	g_object_unref (book_view);
+	bonobo_object_unref (book_view);
 
-	printf ("finished initial population of book view\n");
+	d(printf ("finished initial population of book view\n"));
 }
 
 static void
 e_book_backend_file_stop_book_view (EBookBackend  *backend,
 				    EDataBookView *book_view)
 {
-	FileBackendSearchClosure *closure = g_object_get_data (G_OBJECT (book_view), "EBookBackendFile.BookView::closure");
-	if (!closure) {
-		printf ("book view is already done populating\n");
-		return;
+	FileBackendSearchClosure *closure;
+
+	g_mutex_lock (e_data_book_view_get_mutex (book_view));
+
+	closure = get_closure (book_view);
+
+	if (closure) {
+		d(printf ("stopping running query!\n"));
+		g_mutex_lock (closure->mutex);
+		closure->stopped = TRUE;
+		g_mutex_unlock (closure->mutex);
+	}
+	else {
+		d(printf ("either the query is already finished or hasn't started yet (we won the race)\n"));
+		closure = init_closure (book_view);
+		closure->stopped = TRUE;
 	}
 
-	printf ("stopping book view!\n");
-	g_mutex_lock (closure->mutex);
-	closure->stopped = TRUE;
-	g_mutex_lock (closure->mutex);
+	g_mutex_unlock (e_data_book_view_get_mutex (book_view));
 }
 
 typedef struct {
@@ -881,12 +918,12 @@ e_book_backend_file_load_source (EBookBackend           *backend,
 	} else {
 		db_error = db->open (db, NULL, filename, NULL, DB_HASH, DB_RDONLY, 0666);
 
-		if (db_error != 0) {
+		if (db_error != 0 && !only_if_exists) {
 			int rv;
 
 			/* the database didn't exist, so we create the
 			   directory then the .db */
-			rv = mkdir (dirname, 0777);
+			rv = e_util_mkdir_hier (dirname, 0777);
 			if (rv == -1 && errno != EEXIST) {
 				g_warning ("failed to make directory %s: %s", dirname, strerror (errno));
 				if (errno == EACCES || errno == EPERM)
@@ -897,7 +934,7 @@ e_book_backend_file_load_source (EBookBackend           *backend,
 
 			db_error = db->open (db, NULL, filename, NULL, DB_HASH, DB_CREATE, 0666);
 
-			if (db_error == 0 && !only_if_exists) {
+			if (db_error == 0) {
 				EContact *contact;
 
 				contact = do_create(bf, XIMIAN_VCARD);
