@@ -10,12 +10,15 @@
  */
 
 #include <config.h>
+#include <pthread.h>
 #include <bonobo/bonobo-main.h>
 #include "e-book-view-listener.h"
 #include "e-book-view.h"
 #include "e-contact.h"
 #include "e-book-marshal.h"
 
+#define d(x)
+
 static EBookViewStatus e_book_view_listener_convert_status (GNOME_Evolution_Addressbook_CallStatus status);
 
 enum {
@@ -27,10 +30,58 @@ static guint e_book_view_listener_signals [LAST_SIGNAL];
 
 static BonoboObjectClass          *parent_class;
 
+#define READ_END 0
+#define WRITE_END 1
+
 struct _EBookViewListenerPrivate {
 	guint stopped      : 1;
+	GAsyncQueue *queue;
+
+	GMutex *idle_mutex;
+	guint idle_id;
 };
 
+static void
+free_response (EBookViewListenerResponse *response)
+{
+	g_return_if_fail (response != NULL);
+
+	g_list_foreach (response->ids, (GFunc)g_free, NULL);
+	g_list_free (response->ids);
+	g_list_foreach (response->contacts, (GFunc) g_object_unref, NULL);
+	g_list_free (response->contacts);
+	g_free (response->message);
+	g_free (response);
+}
+
+static gboolean
+main_thread_get_response (gpointer data)
+{
+	EBookViewListener *listener = data;
+	EBookViewListenerResponse *response;
+
+	bonobo_object_ref (listener);
+
+	g_mutex_lock (listener->priv->idle_mutex);
+
+	/* remove the idle call */
+
+	while ((response = g_async_queue_try_pop (listener->priv->queue)) != NULL) {
+
+		g_signal_emit (listener, e_book_view_listener_signals [RESPONSE], 0, response);
+
+		free_response (response);
+	}
+
+	listener->priv->idle_id = -1;
+
+	g_mutex_unlock (listener->priv->idle_mutex);
+
+	bonobo_object_unref (listener);
+       
+	return FALSE;
+}
+
 static void
 e_book_view_listener_queue_response (EBookViewListener         *listener,
 				     EBookViewListenerResponse *response)
@@ -39,17 +90,17 @@ e_book_view_listener_queue_response (EBookViewListener         *listener,
 		return;
 
 	if (listener->priv->stopped) {
-		/* Free response and return */
-		g_list_foreach (response->ids, (GFunc)g_free, NULL);
-		g_list_free (response->ids);
-		g_list_foreach (response->contacts, (GFunc) g_object_unref, NULL);
-		g_list_free (response->contacts);
-		g_free (response->message);
-		g_free (response);
+		free_response (response);
 		return;
 	}
 
-	g_signal_emit (listener, e_book_view_listener_signals [RESPONSE], 0, response);
+	g_mutex_lock (listener->priv->idle_mutex);
+
+	g_async_queue_push (listener->priv->queue, response);
+
+	if (listener->priv->idle_id == -1)
+		listener->priv->idle_id = g_idle_add (main_thread_get_response, listener);
+	g_mutex_unlock (listener->priv->idle_mutex);
 }
 
 /* Add, Remove, Modify */
@@ -146,7 +197,7 @@ impl_BookViewListener_notify_contacts_added (PortableServer_Servant servant,
 {
 	EBookViewListener *listener = E_BOOK_VIEW_LISTENER (bonobo_object (servant));
 
-	printf ("impl_BookViewListener_notify_contacts_added\n");
+	d(printf ("impl_BookViewListener_notify_contacts_added\n"));
 
 	e_book_view_listener_queue_sequence_event (
 		listener, ContactsAddedEvent, vcards);
@@ -159,7 +210,7 @@ impl_BookViewListener_notify_contacts_removed (PortableServer_Servant servant,
 {
 	EBookViewListener *listener = E_BOOK_VIEW_LISTENER (bonobo_object (servant));
 
-	printf ("impl_BookViewListener_notify_contacts_removed\n");
+	d(printf ("impl_BookViewListener_notify_contacts_removed\n"));
 
 	e_book_view_listener_queue_idlist_event (listener, ContactsRemovedEvent, ids);
 }
@@ -171,7 +222,7 @@ impl_BookViewListener_notify_contacts_changed (PortableServer_Servant servant,
 {
 	EBookViewListener *listener = E_BOOK_VIEW_LISTENER (bonobo_object (servant));
 
-	printf ("impl_BookViewListener_notify_contacts_changed\n");
+	d(printf ("impl_BookViewListener_notify_contacts_changed\n"));
 
 	e_book_view_listener_queue_sequence_event (
 		listener, ContactsModifiedEvent, vcards);
@@ -184,7 +235,7 @@ impl_BookViewListener_notify_sequence_complete (PortableServer_Servant servant,
 {
 	EBookViewListener *listener = E_BOOK_VIEW_LISTENER (bonobo_object (servant));
 
-	printf ("impl_BookViewListener_notify_sequence_complete\n");
+	d(printf ("impl_BookViewListener_notify_sequence_complete\n"));
 
 	e_book_view_listener_queue_status_event (listener, SequenceCompleteEvent,
 						 e_book_view_listener_convert_status (status));
@@ -198,7 +249,7 @@ impl_BookViewListener_notify_progress (PortableServer_Servant  servant,
 {
 	EBookViewListener *listener = E_BOOK_VIEW_LISTENER (bonobo_object (servant));
 
-	printf ("impl_BookViewListener_notify_progress\n");
+	d(printf ("impl_BookViewListener_notify_progress\n"));
 
 	e_book_view_listener_queue_message_event (listener, StatusMessageEvent, message);
 }
@@ -223,17 +274,10 @@ e_book_view_listener_convert_status (const GNOME_Evolution_Addressbook_CallStatu
 	}
 }
 
-static void
-e_book_view_listener_construct      (EBookViewListener *listener)
-{
-	/* nothing needed here */
-}
-
 /**
  * e_book_view_listener_new:
- * @book: the #EBookView for which the listener is to be bound
  *
- * Creates and returns a new #EBookViewListener for the book.
+ * Creates and returns a new #EBookViewListener.
  *
  * Returns: a new #EBookViewListener
  */
@@ -243,10 +287,12 @@ e_book_view_listener_new ()
 	EBookViewListener *listener;
 
 	listener = g_object_new (E_TYPE_BOOK_VIEW_LISTENER,
-				 "poa", bonobo_poa_get_threaded (ORBIT_THREAD_HINT_ALL_AT_IDLE, NULL),
+				 "poa", bonobo_poa_get_threaded (ORBIT_THREAD_HINT_PER_REQUEST, NULL),
 				 NULL);
 
-	e_book_view_listener_construct (listener);
+	listener->priv->queue = g_async_queue_new();
+	listener->priv->idle_mutex = g_mutex_new();
+	listener->priv->idle_id = -1;
 
 	return listener;
 }
@@ -270,7 +316,16 @@ e_book_view_listener_dispose (GObject *object)
 {
 	EBookViewListener *listener = E_BOOK_VIEW_LISTENER (object);
 
+	d(printf ("%p: in e_book_view_listener_dispose (%p)\n", pthread_self(), object));
+
 	if (listener->priv) {
+		if (listener->priv->idle_id != -1)
+			g_source_remove (listener->priv->idle_id);
+
+		g_mutex_free (listener->priv->idle_mutex);
+
+		g_async_queue_unref (listener->priv->queue);
+
 		g_free (listener->priv);
 		listener->priv = NULL;
 	}
