@@ -30,6 +30,8 @@
 #include "e-book.h"
 #include "e-name-western.h"
 
+#define d(x)
+
 struct _EContactPrivate {
 };
 
@@ -83,7 +85,7 @@ static void cert_setter (EContact *contact, EVCardAttribute *attr, void *data);
 #define BOOLEAN_FIELD(id,vc,n,pn,ro)  { E_CONTACT_FIELD_TYPE_BOOLEAN, (id), (vc), (n), (pn), (ro) }
 #define LIST_FIELD(id,vc,n,pn,ro)      { E_CONTACT_FIELD_TYPE_LIST, (id), (vc), (n), (pn), (ro) }
 #define MULTI_LIST_FIELD(id,vc,n,pn,ro) { E_CONTACT_FIELD_TYPE_MULTI, (id), (vc), (n), (pn), (ro) }
-#define GETSET_FIELD(id,vc,n,pn,ro,get,set)    { E_CONTACT_FIELD_TYPE_GETSET, (id), (vc), (n), (pn), (ro), -1, NULL, NULL, (get), (set) }
+#define GETSET_FIELD(id,vc,n,pn,ro,get,set)    { E_CONTACT_FIELD_TYPE_STRING | E_CONTACT_FIELD_TYPE_GETSET, (id), (vc), (n), (pn), (ro), -1, NULL, NULL, (get), (set) }
 #define STRUCT_FIELD(id,vc,n,pn,ro,get,set,ty)    { E_CONTACT_FIELD_TYPE_STRUCT | E_CONTACT_FIELD_TYPE_GETSET, (id), (vc), (n), (pn), (ro), -1, NULL, NULL, (get), (set), (ty) }
 #define SYNTH_STR_FIELD(id,n,pn,ro)  { E_CONTACT_FIELD_TYPE_STRING | E_CONTACT_FIELD_TYPE_SYNTHETIC, (id), NULL, (n), (pn), (ro) }
 #define LIST_ELEM_STR_FIELD(id,vc,n,pn,ro,nm) { E_CONTACT_FIELD_TYPE_LIST_ELEM | E_CONTACT_FIELD_TYPE_SYNTHETIC | E_CONTACT_FIELD_TYPE_STRING, (id), (vc), (n), (pn), (ro), (nm) }
@@ -147,18 +149,25 @@ static EContactFieldInfo field_info[] = {
 	BOOLEAN_FIELD        (E_CONTACT_WANTS_HTML, EVC_X_WANTS_HTML, "wants_html", N_("Wants HTML Mail"), FALSE),
 
 	/* Instant messaging fields */
-	MULTI_LIST_FIELD (E_CONTACT_IM_AIM,    EVC_X_AIM,    "im_aim",    N_("AIM Screen Name List"),    FALSE),
-	MULTI_LIST_FIELD (E_CONTACT_IM_JABBER, EVC_X_JABBER, "im_jabber", N_("Jabber Id List"),          FALSE),
-	MULTI_LIST_FIELD (E_CONTACT_IM_YAHOO,  EVC_X_YAHOO,  "im_yahoo",  N_("Yahoo! Screen Name List"), FALSE),
-	MULTI_LIST_FIELD (E_CONTACT_IM_MSN,    EVC_X_MSN,    "im_msn",    N_("MSN Screen Name List"),    FALSE),
-	MULTI_LIST_FIELD (E_CONTACT_IM_ICQ,    EVC_X_ICQ,    "im_icq",    N_("ICQ Id List"),             FALSE),
-
+	MULTI_LIST_FIELD (E_CONTACT_IM_AIM,       EVC_X_AIM,       "im_aim",       N_("AIM Screen Name List"),    FALSE),
+        MULTI_LIST_FIELD (E_CONTACT_IM_GROUPWISE, EVC_X_GROUPWISE, "im_groupwise", N_("Groupwise Id List"),       FALSE),
+	MULTI_LIST_FIELD (E_CONTACT_IM_JABBER, 	  EVC_X_JABBER,    "im_jabber",    N_("Jabber Id List"),          FALSE),
+	MULTI_LIST_FIELD (E_CONTACT_IM_YAHOO,  	  EVC_X_YAHOO,     "im_yahoo",     N_("Yahoo! Screen Name List"), FALSE),
+	MULTI_LIST_FIELD (E_CONTACT_IM_MSN,    	  EVC_X_MSN,       "im_msn",       N_("MSN Screen Name List"),    FALSE),
+	MULTI_LIST_FIELD (E_CONTACT_IM_ICQ,    	  EVC_X_ICQ,       "im_icq",       N_("ICQ Id List"),             FALSE),
+ 
 	ATTR_TYPE_STR_FIELD (E_CONTACT_IM_AIM_HOME_1,    EVC_X_AIM,    "im_aim_home_1",    N_("AIM Home Screen Name 1"),    FALSE, "HOME", 0),
 	ATTR_TYPE_STR_FIELD (E_CONTACT_IM_AIM_HOME_2,    EVC_X_AIM,    "im_aim_home_2",    N_("AIM Home Screen Name 2"),    FALSE, "HOME", 1),
 	ATTR_TYPE_STR_FIELD (E_CONTACT_IM_AIM_HOME_3,    EVC_X_AIM,    "im_aim_home_3",    N_("AIM Home Screen Name 3"),    FALSE, "HOME", 2),
 	ATTR_TYPE_STR_FIELD (E_CONTACT_IM_AIM_WORK_1,    EVC_X_AIM,    "im_aim_work_1",    N_("AIM Work Screen Name 1"),    FALSE, "WORK", 0),
 	ATTR_TYPE_STR_FIELD (E_CONTACT_IM_AIM_WORK_2,    EVC_X_AIM,    "im_aim_work_2",    N_("AIM Work Screen Name 2"),    FALSE, "WORK", 1),
 	ATTR_TYPE_STR_FIELD (E_CONTACT_IM_AIM_WORK_3,    EVC_X_AIM,    "im_aim_work_3",    N_("AIM Work Screen Name 3"),    FALSE, "WORK", 2),
+	ATTR_TYPE_STR_FIELD (E_CONTACT_IM_GROUPWISE_HOME_1, EVC_X_GROUPWISE, "im_groupwise_home_1", N_("Groupwise Home Screen Name 1"),    FALSE, "HOME", 0),
+	ATTR_TYPE_STR_FIELD (E_CONTACT_IM_GROUPWISE_HOME_2, EVC_X_GROUPWISE, "im_groupwise_home_2", N_("Groupwise Home Screen Name 2"),    FALSE, "HOME", 1),
+	ATTR_TYPE_STR_FIELD (E_CONTACT_IM_GROUPWISE_HOME_3, EVC_X_GROUPWISE, "im_groupwise_home_3", N_("Groupwise Home Screen Name 3"),    FALSE, "HOME", 2),
+	ATTR_TYPE_STR_FIELD (E_CONTACT_IM_GROUPWISE_WORK_1, EVC_X_GROUPWISE, "im_groupwise_work_1", N_("Groupwise Work Screen Name 1"),    FALSE, "WORK", 0),
+	ATTR_TYPE_STR_FIELD (E_CONTACT_IM_GROUPWISE_WORK_2, EVC_X_GROUPWISE, "im_groupwise_work_2", N_("Groupwise Work Screen Name 2"),    FALSE, "WORK", 1),
+	ATTR_TYPE_STR_FIELD (E_CONTACT_IM_GROUPWISE_WORK_3, EVC_X_GROUPWISE, "im_groupwise_work_3", N_("Groupwise Work Screen Name 3"),    FALSE, "WORK", 2),
 	ATTR_TYPE_STR_FIELD (E_CONTACT_IM_JABBER_HOME_1, EVC_X_JABBER, "im_jabber_home_1", N_("Jabber Home Id 1"),          FALSE, "HOME", 0),
 	ATTR_TYPE_STR_FIELD (E_CONTACT_IM_JABBER_HOME_2, EVC_X_JABBER, "im_jabber_home_2", N_("Jabber Home Id 2"),          FALSE, "HOME", 1),
 	ATTR_TYPE_STR_FIELD (E_CONTACT_IM_JABBER_HOME_3, EVC_X_JABBER, "im_jabber_home_3", N_("Jabber Home Id 3"),          FALSE, "HOME", 2),
@@ -195,9 +204,9 @@ static EContactFieldInfo field_info[] = {
 	STRING_FIELD    (E_CONTACT_ASSISTANT, EVC_X_ASSISTANT, "assistant", N_("Assistant"),       FALSE),
 
 	/* Web fields */
-	STRING_FIELD (E_CONTACT_HOMEPAGE_URL, EVC_URL,        "homepage_url", N_("Homepage URL"), FALSE),
-	STRING_FIELD (E_CONTACT_BLOG_URL,     EVC_X_BLOG_URL, "blog_url",     N_("Weblog URL"),   FALSE),
-	STRING_FIELD (E_CONTACT_VIDEO_URL,     EVC_X_VIDEO_URL, "video_url",     N_("Video Conferencing URL"),   FALSE),
+	STRING_FIELD (E_CONTACT_HOMEPAGE_URL, EVC_URL,         "homepage_url", N_("Homepage URL"), FALSE),
+	STRING_FIELD (E_CONTACT_BLOG_URL,     EVC_X_BLOG_URL,  "blog_url",     N_("Weblog URL"),   FALSE),
+	STRING_FIELD (E_CONTACT_VIDEO_URL,    EVC_X_VIDEO_URL, "video_url",    N_("Video Conferencing URL"),   FALSE),
 
 	/* Photo/Logo */
 	STRUCT_FIELD    (E_CONTACT_PHOTO, EVC_PHOTO, "photo", N_("Photo"), FALSE, photo_getter, photo_setter, e_contact_photo_get_type),
@@ -207,12 +216,8 @@ static EContactFieldInfo field_info[] = {
 	ATTR_TYPE_STRUCT_FIELD (E_CONTACT_X509_CERT,  EVC_KEY, "x509Cert",  N_("X.509 Certificate"), FALSE, "X509", cert_getter, cert_setter, e_contact_cert_get_type),
 
 	/* Contact categories */
-#if notyet
 	LIST_FIELD      (E_CONTACT_CATEGORY_LIST, EVC_CATEGORIES, "category_list", N_("Category List"), FALSE),
 	SYNTH_STR_FIELD (E_CONTACT_CATEGORIES,                    "categories",    N_("Categories"),    FALSE),
-#else
-	STRING_FIELD    (E_CONTACT_CATEGORIES,    EVC_CATEGORIES, "categories",    N_("Categories"),    FALSE),
-#endif
 
 	/* Collaboration fields */
 	STRING_FIELD (E_CONTACT_CALENDAR_URI, EVC_CALURI,      "caluri",     N_("Calendar URI"),  FALSE),
@@ -342,13 +347,11 @@ e_contact_get_first_attr (EContact *contact, const char *attr_name)
 
 	for (l = attrs; l; l = l->next) {
 		EVCardAttribute *attr = l->data;
-		const char *name, *group;
+		const char *name;
 
-		group = e_vcard_attribute_get_group (attr);
 		name = e_vcard_attribute_get_name (attr);
 
-		/* all the attributes we care about should be in group "" */
-		if ((!group || !*group) && !strcasecmp (name, attr_name))
+		if (!strcasecmp (name, attr_name))
 			return attr;
 	}
 
@@ -365,8 +368,12 @@ photo_getter (EContact *contact, EVCardAttribute *attr)
 
 		if (values && values->data) {
 			GString *s = values->data;
-			EContactPhoto *photo = g_new (EContactPhoto, 1);
+			EContactPhoto *photo;
 
+			if (!s->len)
+				return NULL;
+
+			photo = g_new (EContactPhoto, 1);
 			photo->length = s->len;
 			photo->data = g_malloc (photo->length);
 			memcpy (photo->data, s->str, photo->length);
@@ -385,6 +392,11 @@ photo_setter (EContact *contact, EVCardAttribute *attr, void *data)
 	const char *mime_type;
 	char *image_type = "X-EVOLUTION-UNKNOWN";
 
+	if (!photo)
+		return;
+
+	g_return_if_fail (photo->length > 0);
+
 	e_vcard_attribute_add_param_with_value (attr,
 						e_vcard_attribute_param_new (EVC_ENCODING),
 						"b");
@@ -427,6 +439,11 @@ fn_getter (EContact *contact, EVCardAttribute *attr)
 static void
 fn_setter (EContact *contact, EVCardAttribute *attr, void *data)
 {
+	gchar *name_str = data;
+
+	if (!name_str)
+		return;
+
 	e_vcard_attribute_add_value (attr, (char*)data);
 
 	attr = e_contact_get_first_attr (contact, EVC_N);
@@ -467,6 +484,9 @@ n_setter (EContact *contact, EVCardAttribute *attr, void *data)
 {
 	EContactName *name = data;
 
+	if (!name)
+		return;
+
 	e_vcard_attribute_add_value (attr, name->family);
 	e_vcard_attribute_add_value (attr, name->given);
 	e_vcard_attribute_add_value (attr, name->additional);
@@ -524,6 +544,9 @@ adr_setter (EContact *contact, EVCardAttribute *attr, void *data)
 {
 	EContactAddress *addr = data;
 
+	if (!addr)
+		return;
+
 	e_vcard_attribute_add_value (attr, addr->po);
 	e_vcard_attribute_add_value (attr, addr->ext);
 	e_vcard_attribute_add_value (attr, addr->street);
@@ -540,7 +563,12 @@ date_getter (EContact *contact, EVCardAttribute *attr)
 {
 	if (attr) {
 		GList *p = e_vcard_attribute_get_values (attr);
-		EContactDate *date = e_contact_date_from_string (p && p->data ? (char*)p->data : "");
+		EContactDate *date;
+
+		if (p && p->data && ((gchar *) p->data) [0])
+			date = e_contact_date_from_string ((char *) p->data);
+		else
+			date = NULL;
 
 		return date;
 	}
@@ -552,7 +580,12 @@ static void
 date_setter (EContact *contact, EVCardAttribute *attr, void *data)
 {
 	EContactDate *date = data;
-	char *str = e_contact_date_to_string (date);
+	char *str;
+
+	if (!date)
+		return;
+
+	str = e_contact_date_to_string (date);
 
 	e_vcard_attribute_add_value (attr, str);
 	g_free (str);
@@ -593,6 +626,9 @@ cert_setter (EContact *contact, EVCardAttribute *attr, void *data)
 {
 	EContactCert *cert = data;
 
+	if (!cert)
+		return;
+
 	e_vcard_attribute_add_param_with_value (attr,
 						e_vcard_attribute_param_new (EVC_ENCODING),
 						"b");
@@ -660,14 +696,12 @@ e_contact_set_property (GObject *object,
 			GList *l;
 
 			for (l = attrs; l; l = l->next) {
-				const char *name, *group;
+				const char *name;
 
 				attr = l->data;
-				group = e_vcard_attribute_get_group (attr);
 				name = e_vcard_attribute_get_name (attr);
 
-				/* all the attributes we care about should be in group "" */
-				if ((!group || !*group) && !strcasecmp (name, info->vcard_field_name)) {
+				if (!strcasecmp (name, info->vcard_field_name)) {
 					if (num_left-- == 0) {
 						found = TRUE;
 						break;
@@ -701,18 +735,16 @@ e_contact_set_property (GObject *object,
 			GList *l;
 
 			for (l = attrs; l && !found; l = l->next) {
-				const char *name, *group;
+				const char *name;
 				gboolean found_needed1, found_needed2;
 
 				found_needed1 = (info->attr_type1 == NULL);
 				found_needed2 = (info->attr_type2 == NULL);
 
 				attr = l->data;
-				group = e_vcard_attribute_get_group (attr);
 				name = e_vcard_attribute_get_name (attr);
 
-				/* all the attributes we care about should be in group "" */
-				if ((!group || !*group) && !strcasecmp (name, info->vcard_field_name)) {
+				if (!strcasecmp (name, info->vcard_field_name)) {
 					GList *params;
 
 					for (params = e_vcard_attribute_get_params (attr); params; params = params->next) {
@@ -770,7 +802,7 @@ e_contact_set_property (GObject *object,
 			}
 
 			if (info->t & E_CONTACT_FIELD_TYPE_STRUCT || info->t & E_CONTACT_FIELD_TYPE_GETSET) {
-				void *data = info->t & E_CONTACT_FIELD_TYPE_STRUCT ? g_value_get_boxed (value) : g_value_get_pointer (value);
+				void *data = info->t & E_CONTACT_FIELD_TYPE_STRUCT ? g_value_get_boxed (value) : (char*)g_value_get_string (value);
 
 				info->struct_setter (contact, attr, data);
 			}
@@ -793,7 +825,7 @@ e_contact_set_property (GObject *object,
 				if (!sval || !*sval)
 					return;
 
-				printf ("adding new %s\n", info->vcard_field_name);
+				d(printf ("adding new %s\n", info->vcard_field_name));
 
 				attr = e_vcard_attribute_new (NULL, info->vcard_field_name);
 				e_vcard_add_attribute (E_VCARD (contact), attr);
@@ -817,26 +849,66 @@ e_contact_set_property (GObject *object,
 			}
 
 		}
+		else {
+			switch (info->field_id) {
+			case E_CONTACT_CATEGORIES: {
+				EVCardAttribute *attr = e_contact_get_first_attr (contact, EVC_CATEGORIES);
+				char **split, **s;
+				const char *str;
+
+				if (attr)
+					e_vcard_attribute_remove_values (attr);
+				else {
+					/* we didn't find it - add a new attribute */
+					attr = e_vcard_attribute_new (NULL, EVC_CATEGORIES);
+					e_vcard_add_attribute (E_VCARD (contact), attr);
+				}
+
+				str = g_value_get_string (value);
+				if (str) {
+					split = g_strsplit (str, ",", 0);
+					if (split) {
+						for (s = split; *s; s++) {
+							e_vcard_attribute_add_value (attr, g_strstrip (*s));
+						}
+						g_strfreev (split);
+					} else
+						e_vcard_attribute_add_value (attr, str);
+				}
+				break;
+			}
+			default:
+				g_warning ("unhandled synthetic field 0x%02x", info->field_id);
+				break;
+			}
+		}
 	}
 	else if (info->t & E_CONTACT_FIELD_TYPE_STRUCT || info->t & E_CONTACT_FIELD_TYPE_GETSET) {
 		EVCardAttribute *attr = e_contact_get_first_attr (contact, info->vcard_field_name);
-		void *data = info->t & E_CONTACT_FIELD_TYPE_STRUCT ? g_value_get_boxed (value) : g_value_get_pointer (value);
+		void *data = info->t & E_CONTACT_FIELD_TYPE_STRUCT ? g_value_get_boxed (value) : (char*)g_value_get_string (value);
 
-		if (attr) {
-			printf ("overwriting existing %s\n", info->vcard_field_name);
+		if (attr && data) {
+			d(printf ("overwriting existing %s\n", info->vcard_field_name));
 			/* remove all existing values and parameters.
 			   the setter will add the correct ones */
 			e_vcard_attribute_remove_values (attr);
 			e_vcard_attribute_remove_params (attr);
+
+			info->struct_setter (contact, attr, data);
 		}
-		else {
-			printf ("adding new %s\n", info->vcard_field_name);
+		else if (data) {
+			d(printf ("adding new %s\n", info->vcard_field_name));
 			attr = e_vcard_attribute_new (NULL, info->vcard_field_name);
 
 			e_vcard_add_attribute (E_VCARD (contact), attr);
-		}
 
-		info->struct_setter (contact, attr, data);
+			info->struct_setter (contact, attr, data);
+		}
+		else if (attr) {
+			d(printf ("removing %s\n", info->vcard_field_name));
+
+			e_vcard_remove_attribute (E_VCARD (contact), attr);
+		}
 	}
 	else if (info->t & E_CONTACT_FIELD_TYPE_BOOLEAN) {
 		EVCardAttribute *attr;
@@ -844,7 +916,7 @@ e_contact_set_property (GObject *object,
 		/* first we search for an attribute we can overwrite */
 		attr = e_contact_get_first_attr (contact, info->vcard_field_name);
 		if (attr) {
-			printf ("setting %s to `%s'\n", info->vcard_field_name, g_value_get_string (value));
+			d(printf ("setting %s to `%s'\n", info->vcard_field_name, g_value_get_string (value)));
 			e_vcard_attribute_remove_values (attr);
 			e_vcard_attribute_add_value (attr, g_value_get_boolean (value) ? "TRUE" : "FALSE");
 		}
@@ -862,7 +934,7 @@ e_contact_set_property (GObject *object,
 		/* first we search for an attribute we can overwrite */
 		attr = e_contact_get_first_attr (contact, info->vcard_field_name);
 		if (attr) {
-			printf ("setting %s to `%s'\n", info->vcard_field_name, sval);
+			d(printf ("setting %s to `%s'\n", info->vcard_field_name, sval));
 			e_vcard_attribute_remove_values (attr);
 			if (sval)
 				e_vcard_attribute_add_value (attr, sval);
@@ -901,32 +973,6 @@ e_contact_set_property (GObject *object,
 	}
 }
 
-static GList *
-e_contact_get_email_list (EContact *contact)
-{
-	GList *rv = NULL;
-	GList *attrs, *l;
-
-	attrs = e_vcard_get_attributes (E_VCARD (contact));
-
-	for (l = attrs; l; l = l->next) {
-		EVCardAttribute *attr = l->data;
-		const char *name, *group;
-
-		group = e_vcard_attribute_get_group (attr);
-		name = e_vcard_attribute_get_name (attr);
-
-		/* all the attributes we care about should be in group "" */
-		if ((!group || !*group) && !strcasecmp (name, EVC_EMAIL)) {
-			GList *v = e_vcard_attribute_get_values (attr);
-
-			rv = g_list_append (rv, v ? g_strdup (v->data) : NULL);
-		}
-	}
-
-	return rv;
-}
-
 static EVCardAttribute *
 e_contact_find_attribute_with_types (EContact *contact, const char *attr_name, const char *type_needed1, const char *type_needed2, int nth)
 {
@@ -937,16 +983,14 @@ e_contact_find_attribute_with_types (EContact *contact, const char *attr_name, c
 
 	for (l = attrs; l; l = l->next) {
 		EVCardAttribute *attr = l->data;
-		const char *name, *group;
+		const char *name;
 
 		found_needed1 = (type_needed1 == NULL);
 		found_needed2 = (type_needed2 == NULL);
 
-		group = e_vcard_attribute_get_group (attr);
 		name = e_vcard_attribute_get_name (attr);
 
-		/* all the attributes we care about should be in group "" */
-		if ((!group || !*group) && !strcasecmp (name, attr_name)) {
+		if (!strcasecmp (name, attr_name)) {
 			GList *params;
 
 			for (params = e_vcard_attribute_get_params (attr); params; params = params->next) {
@@ -1033,8 +1077,14 @@ e_contact_get_property (GObject *object,
 	else if (info->t & E_CONTACT_FIELD_TYPE_LIST) {
 		EVCardAttribute *attr = e_contact_get_first_attr (contact, info->vcard_field_name);
 
-		if (attr)
-			g_value_set_pointer (value, e_vcard_attribute_get_values (attr));
+		if (attr) {
+			GList *list = g_list_copy (e_vcard_attribute_get_values (attr));
+			GList *l;
+			for (l = list; l; l = l->next)
+				l->data = g_strdup (l->data);
+
+			g_value_set_pointer (value, list);
+		}
 	}
 	else if (info->t & E_CONTACT_FIELD_TYPE_LIST_ELEM) {
 		if (info->t & E_CONTACT_FIELD_TYPE_STRING) {
@@ -1044,13 +1094,11 @@ e_contact_get_property (GObject *object,
 
 			for (l = attrs; l; l = l->next) {
 				EVCardAttribute *attr = l->data;
-				const char *name, *group;
+				const char *name;
 
-				group = e_vcard_attribute_get_group (attr);
 				name = e_vcard_attribute_get_name (attr);
 
-				/* all the attributes we care about should be in group "" */
-				if ((!group || !*group) && !strcasecmp (name, info->vcard_field_name)) {
+				if (!strcasecmp (name, info->vcard_field_name)) {
 					GList *v;
 					int count;
 
@@ -1073,13 +1121,11 @@ e_contact_get_property (GObject *object,
 
 			for (l = attrs; l; l = l->next) {
 				EVCardAttribute *attr = l->data;
-				const char *name, *group;
+				const char *name;
 
-				group = e_vcard_attribute_get_group (attr);
 				name = e_vcard_attribute_get_name (attr);
 
-				/* all the attributes we care about should be in group "" */
-				if ((!group || !*group) && !strcasecmp (name, info->vcard_field_name)) {
+				if (!strcasecmp (name, info->vcard_field_name)) {
 					if (num_left-- == 0) {
 						GList *v = e_vcard_attribute_get_values (attr);
 
@@ -1127,7 +1173,11 @@ e_contact_get_property (GObject *object,
 		if (attr)
 			rv = info->struct_getter (contact, attr);
 
-		g_value_set_pointer (value, rv);
+		if (info->t & E_CONTACT_FIELD_TYPE_STRUCT)
+			g_value_set_boxed (value, rv);
+		else
+			g_value_set_string (value, (char*)rv);
+			
 	}
 
 	else if (info->t & E_CONTACT_FIELD_TYPE_SYNTHETIC) {
@@ -1152,8 +1202,30 @@ e_contact_get_property (GObject *object,
 			g_value_set_string (value, str);
 			break;
 		}
+		case E_CONTACT_CATEGORIES: {
+			EVCardAttribute *attr = e_contact_get_first_attr (contact, EVC_CATEGORIES);
+			char *rv = NULL;
+
+			if (attr) {
+				GString *str = g_string_new ("");
+				GList *v = e_vcard_attribute_get_values (attr);
+				while (v) {
+					g_string_append (str, (char*)v->data);
+					v = v->next;
+					if (v)
+						g_string_append_c (str, ',');
+				}
+
+				rv = g_string_free (str, FALSE);
+			}
+
+			g_value_set_string (value, rv);
+			g_free (rv);
+			break;
+		}
 		default:
 			g_warning ("unhandled synthetic field 0x%02x", info->field_id);
+			break;
 		}
 	}
 	else {
@@ -1164,13 +1236,11 @@ e_contact_get_property (GObject *object,
 
 		for (l = attrs; l; l = l->next) {
 			EVCardAttribute *attr = l->data;
-			const char *name, *group;
+			const char *name;
 
-			group = e_vcard_attribute_get_group (attr);
 			name = e_vcard_attribute_get_name (attr);
 
-			/* all the attributes we care about should be in group "" */
-			if ((!group || !*group) && !strcasecmp (name, info->vcard_field_name)) {
+			if (!strcasecmp (name, info->vcard_field_name)) {
 				GList *v;
 				v = e_vcard_attribute_get_values (attr);
 
@@ -1198,8 +1268,11 @@ e_contact_new (void)
 EContact*
 e_contact_new_from_vcard  (const char *vcard)
 {
-	EContact *contact = g_object_new (E_TYPE_CONTACT, NULL);
+	EContact *contact;
 
+	g_return_val_if_fail (vcard != NULL, NULL);
+
+	contact = g_object_new (E_TYPE_CONTACT, NULL);
 	e_vcard_construct (E_VCARD (contact), vcard);
 
 	return contact;
@@ -1208,9 +1281,13 @@ e_contact_new_from_vcard  (const char *vcard)
 EContact*
 e_contact_duplicate (EContact *contact)
 {
-	char *vcard = e_vcard_to_string (E_VCARD (contact), EVC_FORMAT_VCARD_30);
-	EContact *c = e_contact_new_from_vcard (vcard);
+	char *vcard;
+	EContact *c;
 
+	g_return_val_if_fail (E_IS_CONTACT (contact), NULL);
+
+	vcard = e_vcard_to_string (E_VCARD (contact), EVC_FORMAT_VCARD_30);
+	c = e_contact_new_from_vcard (vcard);
 	g_free (vcard);
 
 	return c;
@@ -1248,6 +1325,22 @@ e_contact_pretty_name (EContactField field_id)
 	return "";
 }
 
+const char*
+e_contact_vcard_attribute  (EContactField field_id)
+{
+	int i;
+
+	g_return_val_if_fail (field_id >= 1 && field_id <= E_CONTACT_FIELD_LAST, "");
+
+	for (i = 0; i < G_N_ELEMENTS (field_info); i ++) {
+		if (field_id == field_info[i].field_id)
+			return field_info[i].vcard_field_name;
+	}
+
+	g_warning ("unknown field id %d", field_id);
+	return NULL;
+}
+
 EContactField
 e_contact_field_id (const char *field_name)
 {
@@ -1301,7 +1394,7 @@ e_contact_get_const (EContact *contact, EContactField field_id)
 void
 e_contact_set (EContact *contact, EContactField field_id, gpointer value)
 {
-	printf ("e_contact_set (%p, %d, %p)\n", contact, field_id, value);
+	d(printf ("e_contact_set (%p, %d, %p)\n", contact, field_id, value));
 
 	g_return_if_fail (contact && E_IS_CONTACT (contact));
 	g_return_if_fail (field_id >= 1 && field_id <= E_CONTACT_FIELD_LAST);
@@ -1311,6 +1404,74 @@ e_contact_set (EContact *contact, EContactField field_id, gpointer value)
 		      NULL);
 }
 
+GList*
+e_contact_get_attributes (EContact *contact, EContactField field_id)
+{
+	GList *l = NULL;
+	GList *attrs, *a;
+	int i;
+	EContactFieldInfo *info = NULL;
+
+	g_return_val_if_fail (contact && E_IS_CONTACT (contact), NULL);
+	g_return_val_if_fail (field_id >= 1 && field_id <= E_CONTACT_FIELD_LAST, NULL);
+
+	for (i = 0; i < G_N_ELEMENTS (field_info); i++) {
+		if (field_info[i].field_id == field_id) {
+			info = &field_info[i];
+			break;
+		}
+	}
+
+	if (!info) {
+		g_warning ("unknown field %d", field_id);
+		return NULL;
+	}
+
+	attrs = e_vcard_get_attributes (E_VCARD (contact));
+
+	for (a = attrs; a; a = a->next) {
+		EVCardAttribute *attr = a->data;
+		const char *name;
+
+		name = e_vcard_attribute_get_name (attr);
+
+		if (!strcasecmp (name, info->vcard_field_name)) {
+			l = g_list_append (l, e_vcard_attribute_copy (attr));
+		}
+	}
+
+	return l;
+}
+
+void
+e_contact_set_attributes (EContact *contact, EContactField field_id, GList *attributes)
+{
+	EContactFieldInfo *info = NULL;
+	GList *l;
+	int i;
+
+	g_return_if_fail (contact && E_IS_CONTACT (contact));
+	g_return_if_fail (field_id >= 1 && field_id <= E_CONTACT_FIELD_LAST);
+
+	for (i = 0; i < G_N_ELEMENTS (field_info); i++) {
+		if (field_info[i].field_id == field_id) {
+			info = &field_info[i];
+			break;
+		}
+	}
+
+	if (!info) {
+		g_warning ("unknown field %d", field_id);
+		return;
+	}
+
+	e_vcard_remove_attributes (E_VCARD (contact), NULL, info->vcard_field_name);
+
+	for (l = attributes; l; l = l->next)
+		e_vcard_add_attribute (E_VCARD (contact),
+				       e_vcard_attribute_copy ((EVCardAttribute*)l->data));
+}
+
 EContactName*
 e_contact_name_new ()
 {
@@ -1344,6 +1505,8 @@ e_contact_name_from_string (const char *name_str)
 	EContactName *name = e_contact_name_new();
 	ENameWestern *western;
 
+	g_return_val_if_fail (name_str != NULL, NULL);
+
 	western = e_name_western_parse (name_str ? name_str : "");
 	
 	name->prefixes   = g_strdup (western->prefix);
@@ -1360,7 +1523,11 @@ e_contact_name_from_string (const char *name_str)
 EContactName*
 e_contact_name_copy (EContactName *n)
 {
-	EContactName *name = e_contact_name_new();
+	EContactName *name;
+
+	g_return_val_if_fail (n != NULL, NULL);
+
+	name = e_contact_name_new();
 	
 	name->prefixes   = g_strdup (n->prefixes);
 	name->given      = g_strdup (n->given);
@@ -1401,9 +1568,12 @@ e_contact_name_get_type (void)
 EContactDate*
 e_contact_date_from_string (const char *str)
 {
-	EContactDate* date = e_contact_date_new();
+	EContactDate* date;
 	int length;
 
+	g_return_val_if_fail (str != NULL, NULL);
+
+	date = e_contact_date_new();
 	length = strlen(str);
 	
 	if (length == 10 ) {
